#!/usr/bin/perl -w
# $Id: texescaper.pl 2625 2010-02-28 22:59:23Z michalr $

use strict;
use utf8;

binmode(STDIN,"utf8");
binmode(STDOUT,"utf8");
binmode(STDERR,"utf8");


my $utf2tex_hash;
my $utf2tex_hash_file;

sub utf2tex_load_hash {
        my($filename) = @_;
        my $hash;
        open(MAP,"<:utf8",$filename) or die "$filename: $!";
        while(<MAP>) {
                $hash->{$1} = $2 if(/^(.*)\t(.*)/);
        }
        close(MAP);
        return $hash;
}

sub utf2tex_substitute {
    my($str,$hash) = @_;
    my $tex = "";
    $tex .= $hash->{$_} || $_ for(split//,$str);
    return $tex;
}

sub utf2tex_splitatpairend {

    my($str,$openingstr,$closingstr) = @_;
    my($math, $rest) = ("", "");
    my @tmp = split(/\Q$openingstr\E/, substr($str,0,index($str,$closingstr)));
    my $depth = @tmp;

    for (my $i = 0; $i < ($depth - 1); $i++) {
       $math .= substr($str,0,index($str,$closingstr)+length($closingstr),""); 
       my @tmp = split(/\Q$openingstr\E/, substr($str,0,index($str,$closingstr)));
       $depth += (@tmp - 1);
    }
    $math .= substr($str,0,index($str,$closingstr),""); 
    $rest = substr($str,length($closingstr));

    return ($math, $rest);

}

sub utf2tex {

    my($str) = @_;
    my $orig = $str;
    my $out = "";


    # Hack!
    if ($str) {
	$str =~ s/\$\$/!!!!DOUBLEDOLLARHACK!!!!/g;
	$str =~ s/\$/!!!!SINGLEDOLLARHACK!!!!/g;
    }

    $utf2tex_hash = utf2tex_load_hash($utf2tex_hash_file) unless($utf2tex_hash);
    my %mathpairs = (
        '!!!!SINGLEDOLLARHACK!!!!' => '!!!!SINGLEDOLLARHACK!!!!',
        '\(' => '\)',
        '\begin{math}' => '\end{math}',
        '!!!!DOUBLEDOLLARHACK!!!!' => '!!!!DOUBLEDOLLARHACK!!!!',
        '\[' => '\]',
        '\begin{displaymath}' => '\end{displaymath}',
        '\begin{equation}' => '\end{equation}',
        '\begin{equation*}' => '\end{equation*}',
        '\begin{eqnarray}' => '\end{eqnarray}',
        '\begin{eqnarray*}' => '\end{eqnarray*}',
        '\begin{align}' => '\end{align}',
        '\begin{align*}' => '\end{align*}',
        '\begin{alignat}' => '\end{alignat}',
        '\begin{alignat*}' => '\end{alignat*}',
        '\begin{flalign}' => '\end{flalign}',
        '\begin{flalign*}' => '\end{flalign*}',
        '\begin{gather}' => '\end{gather}',
        '\begin{gather*}' => '\end{gather*}',
        '\begin{multline}' => '\end{multline}',
        '\begin{multline*}' => '\end{multline*}');

    while($str) {

        my($begin, $end, $math, $rest) = ("", "", "", "");

        if ($str =~ /\Q!!!!SINGLEDOLLARHACK!!!!\E|\Q\(\E|\Q\begin{math}\E|\!!!!DOUBLEDOLLARHACK!!!!\E|\Q\[\E|\Q\begin{displaymath}\E|\Q\begin{equation}\E|\Q\begin{equation*}\E|\Q\begin{eqnarray}\E|\Q\begin{eqnarray*}\E|\Q\begin{align}\E|\Q\begin{align*}\E|\Q\begin{alignat}\E|\Q\begin{alignat*}\E|\Q\begin{flalign}\E|\Q\begin{flalign*}\E|\Q\begin{gather}\E|\Q\begin{gather*}\E|\Q\begin{multline}\E|\Q\begin{multline*}\E/) {
                my $key = $&;

                ($begin, $end) = split(/\Q$key\E/, $str, 2);
                if ($end) {
                        $out .= utf2tex_substitute($begin,$utf2tex_hash);
                        $out .= "$key";
                        ($math, $rest) = utf2tex_splitatpairend($end,
                                                $key,
                                                $mathpairs{$key});
                        $out .= $math;
                        $out .= $mathpairs{$key};
                        $str = $rest;
                        next;
                }
        }

        $out .= utf2tex_substitute($str,$utf2tex_hash);
        $str = "";

    }

    # Hack!
    $out =~ s/!!!!DOUBLEDOLLARHACK!!!!/\$\$/g;
    $out =~ s/!!!!SINGLEDOLLARHACK!!!!/\$/g;

    return $out;

}

# Main
if ($ARGV[0] and -f $ARGV[0]) {
	$utf2tex_hash_file = $ARGV[0];
} else {
	print STDERR "Usage: $0 <map_file>\n";
	exit 1;
}

my $input;
while (my $line = <STDIN>) {
        $input .= $line;
};
my $output = utf2tex($input);

print '%&program=xelatex
%&encoding=UTF-8 Unicode
\newdimen\vsy
\setlength\vsy{297mm}
\def\toptitle{toptitle}
\def\mainauthors{mainauthors}
\documentclass{validator}
\usepackage{fontspec}
\usepackage{xunicode}
\usepackage{xltxtra}
\usepackage{amsmath}
\usepackage{amsfonts}
\defaultfontfeatures{Mapping=tex-text}
\begin{document}
';

print "$output";

print '
\end{document}
';
